unit uadvdmx;

interface

const
  nInput  = 245; // excluding start code
  nOutput = 255; // excluding start code

function resync(port: Word): Boolean; cdecl;
{
  Uses 256 writes.
}
function reset(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function clearAll(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function clearInput(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function clearOutput(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function defaults(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function setBreakTime(port: Word; value: Byte): Boolean; cdecl;
{
  Uses 2 writes.
}
function setMabTime(port: Word; value: Byte): Boolean; cdecl;
{
  Uses 2 writes.
}
function setOutputCount(port: Word; value: Byte): Boolean; cdecl;
{
  Uses 2 writes.
}
function setMuxAddress(port: Word; value: Byte): Boolean; cdecl;
{
  Uses 2 writes.
}
function enableInputWildcard(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function disableInputWildcard(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function setInputStartcode(port: Word; value: Byte): Boolean; cdecl;
{
  Uses 2 writes.
}
function enableInput(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function disableInput(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function enableOutput(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function disableOutput(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function getInputStartcode(port: Word; var inputStartcode: Byte): Boolean; cdecl;
{
  Uses 2 writes.
}
function getInputCount(port: Word; var inputCount: SmallInt): Boolean; cdecl;
{
  Uses 2 writes.
}
function getTimeSinceInput(port: Word; var timeSinceInput: Word): Boolean; cdecl;
{
  Uses 4 writes.
  In order to convert value to seconds, it must be multiplied by 0.000128.
}
function getMuxValue(port: Word; var muxValue: Byte): Boolean; cdecl;
{
  Uses 2 writes.
}
function input(port: Word; channel: Byte; var value: Byte): Boolean; cdecl;
{
  Uses 3 writes.
  1 input call takes about 0,090 ms.
}
function bulkInput(port: Word; channels: array of Byte; var values: array of Byte): Boolean; cdecl;
{
  Uses 1 + (number of requested input channels * 2) + 1 writes.
}
function setOutputStartcode(port: Word; value: Byte): Boolean; cdecl;
{
  Uses 2 writes.
}
function output(port: Word; channel: Byte; value: Byte): Boolean; cdecl;
{
  Uses 3 writes.
}
function bulkOutput(port: Word; channels: array of Byte; values: array of Byte): Boolean; cdecl;
{
  Uses 1 + (number of requested output channels * 2) + 1 writes.
}
function setMassOutputLimits(port: Word; minChannel, maxChannel: Byte): Boolean; cdecl;
{
  Uses 3 writes.
}
function massOutput(port: Word; values: array of Byte): Boolean; cdecl;
{
  Uses 1 + (number of configured mass output channels) writes.
}
function enableMux(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function disableMux(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function forceMuxOff(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function noForceMuxOff(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function invertedMux(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function nonInvertedMux(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function enableMerge(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function disableMerge(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function finish(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function readEEPROM(port: Word): Boolean; cdecl;
{
  Uses 1 write.
}
function writeEEPROM(port: Word; inputEnabled, outputEnabled, muxEnabled, mergeEnabled: Boolean): Boolean; cdecl;
{
  Uses 5 writes.
}
function echo(port: Word; echoRequest: Byte; var echoResult: Byte): Boolean; cdecl;
{
  Uses 3 writes.
}

implementation

uses
  Windows,
  uparport,
  Dialogs, //!
  Sysutils; //!

const
  LPT_TIMEOUT = 200000;

  COMMAND_SYNC                   = #255;
  COMMAND_RESET                  = '-';
  COMMAND_SET_MUX_ADDRESS        = 'A';
  COMMAND_CLEAR_ALL              = 'C';
  COMMAND_CLEAR_INPUT            = '8';
  COMMAND_CLEAR_OUTPUT           = '9';
  COMMAND_DEFAULTS               = 'P';
  COMMAND_SET_BREAK_TIME         = 'B';
  COMMAND_SET_MAB_TIME           = 'M';
  COMMAND_SET_OUTPUT_COUNT       = 'L';
  COMMAND_ENABLE_INPUT_WILDCARD  = '*';
  COMMAND_DISABLE_INPUT_WILDCARD = '#';
  COMMAND_GET_INPUT_STARTCODE    = 'K';
  COMMAND_GET_INPUT_COUNT        = 'G';
  COMMAND_GET_TIME_SINCE_INPUT   = 'Z';
  COMMAND_GET_MUX_VALUE          = 'Y';
  COMMAND_SET_INPUT_STARTCODE    = 'S';
  COMMAND_ENABLE_INPUT           = 'R';
  COMMAND_DISABLE_INPUT          = 'r';
  COMMAND_ENABLE_OUTPUT          = 'T';
  COMMAND_DISABLE_OUTPUT         = 't';
  COMMAND_INPUT                  = 'i';
  COMMAND_BULK_INPUT             = 'I';
  COMMAND_SET_OUTPUT_STARTCODE   = 'D';
  COMMAND_OUTPUT                 = 'o';
  COMMAND_BULK_OUTPUT            = 'O';
  COMMAND_SET_MASS_OUTPUT_LIMITS = '1';
  COMMAND_MASS_OUTPUT            = 'Q';
  COMMAND_ENABLE_MUX             = 'E';
  COMMAND_DISABLE_MUX            = 'e';
  COMMAND_FORCE_MUX_OFF          = 'F';
  COMMAND_NO_FORCE_MUX_OFF       = 'f';
  COMMAND_INVERTED_MUX           = 'N';
  COMMAND_NON_INVERTED_MUX       = 'n';
  COMMAND_FINISH                 = '^';
  COMMAND_ENABLE_MERGE           = 'X';
  COMMAND_DISABLE_MERGE          = 'x';
  COMMAND_READ_EEPROM            = 'V';
  COMMAND_WRITE_EEPROM           = 'W';
  COMMAND_ECHO                   = '%';

function getBusy(port: Word): Boolean;
begin
  Result := (parIn(port + STATUS) and BUSY) <> BUSY;
end;

procedure setStrobe(port: Word; s: Boolean);
begin
  if s then
    parOut(port + CONTROL, parIn(port + CONTROL) or STR)
  else
    parOut(port + CONTROL, parIn(port + CONTROL) and not(STR))
end;

function getNibble(port: Word): Byte;
begin
  Result := 0;
  if parIn(port + STATUS) and ACK = ACK then
    Result := Result or 8;
  if parIn(port + STATUS) and PE  = PE then
    Result := Result or 4;
  if parIn(port + STATUS) and SEL = SEL then
    Result := Result or 2;
  if parIn(port + STATUS) and ERR = ERR then
    Result := Result or 1;
end;

function writeChar(port: Word; c: Char; var nibble: Byte): Boolean;
var
  timeOut: Cardinal;
begin
  // make sure that STR is low
  setStrobe(port, False);
  // check that _BUSY is low
//  if not(getBusy(port)) then {!!!!!!!!!!!!!!!!!!!!!!}
//  begin
    // put data on DATA port
    parOut(port + DATA, Ord(c));
    // make STR high
    setStrobe(port, True);
    // wait until _BUSY is high, indicating that the device recognized the strobe
    timeOut := LPT_TIMEOUT;
    while not(getBusy(port)) and (timeOut > 0) do
      Dec(timeOut); // do nothing
    if getBusy(port) then
    begin
      // make STR low
      setStrobe(port, False);
      // wait until _BUSY is low again, indicating that the device has finished processing
      timeOut := LPT_TIMEOUT;
      while getBusy(port) and (timeOut > 0) do
        Dec(timeOut); // do nothing
      if not(getBusy(port)) then
      begin
        // retrieve any returned data from ACK, PE, SEL and ERR
        nibble := getNibble(port);
        Result := True;
      end
      else
        Result := False;
    end
    else
      Result := False;
//  end
//  else
//    Result := False;
end;

function resync(port: Word): Boolean;
var
  dummy, i: Byte;
begin
  setStrobe(port, False);
  for i := 0 to 255 do
  begin
    Result := writeChar(port, #255, dummy);
    if not(Result) then
      Break;
  end;
end;

function reset(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_RESET, dummy);
end;

function clearAll(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_CLEAR_ALL, dummy);
end;

function clearInput(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_CLEAR_INPUT, dummy);
end;

function clearOutput(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_CLEAR_OUTPUT, dummy);
end;

function defaults(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_DEFAULTS, dummy);
end;

function setBreakTime(port: Word; value: Byte): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_SET_BREAK_TIME, dummy);
  if Result then
    Result := writeChar(port, Chr(value), dummy);
end;

function setMabTime(port: Word; value: Byte): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_SET_MAB_TIME, dummy);
  if Result then
    Result := writeChar(port, Chr(value), dummy);
end;

function setOutputCount(port: Word; value: Byte): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_SET_OUTPUT_COUNT, dummy);
  if Result then
    Result := writeChar(port, Chr(value), dummy);
end;

function setMuxAddress(port: Word; value: Byte): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_SET_MUX_ADDRESS, dummy);
  if Result then
    Result := writeChar(port, Chr(value), dummy);
end;

function enableInputWildcard(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_ENABLE_INPUT_WILDCARD, dummy);
end;

function disableInputWildcard(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_DISABLE_INPUT_WILDCARD, dummy);
end;

function setInputStartcode(port: Word; value: Byte): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_SET_INPUT_STARTCODE, dummy);
  if Result then
    Result := writeChar(port, Chr(value), dummy);
end;

function enableInput(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_ENABLE_INPUT, dummy);
end;

function disableInput(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_DISABLE_INPUT, dummy);
end;

function enableOutput(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_ENABLE_OUTPUT, dummy);
end;

function disableOutput(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_DISABLE_OUTPUT, dummy);
end;

function getInputStartcode(port: Word; var inputStartcode: Byte): Boolean;
var
  nibble_low, nibble_high: Byte;
begin
  Result := writeChar(port, COMMAND_GET_INPUT_STARTCODE, nibble_low);
  if Result then
  begin
    Result := writeChar(port, COMMAND_SYNC, nibble_high);
    inputStartcode := (nibble_high shl 4) or nibble_low;
  end;
end;

function getInputCount(port: Word; var inputCount: SmallInt): Boolean;
var
  nibble_low, nibble_high: Byte;
begin
  Result := writeChar(port, COMMAND_GET_INPUT_COUNT, nibble_low);
  if Result then
  begin
    Result := writeChar(port, COMMAND_SYNC, nibble_high);
    inputCount := (nibble_high shl 4) or nibble_low;
    // a value of 255 indicates that a break was received after the
    // previous break without even a start code being received inbetween.
    // a value of 0 would mean that no channel values were received,
    // but a start code was received.
    if inputCount = 255 then
      inputCount := -1;
  end;
end;

function getTimeSinceInput(port: Word; var timeSinceInput: Word): Boolean;
var
  nibble_low, nibble_high: Byte;
  byte_high, byte_low: Byte;
begin
  Result := writeChar(port, COMMAND_GET_TIME_SINCE_INPUT, nibble_low);
  if Result then
  begin
    Result := writeChar(port, COMMAND_SYNC, nibble_high);
    if Result then
    begin
      byte_high := (nibble_high shl 4) or nibble_low;
      Result := writeChar(port, COMMAND_SYNC, nibble_low);
      if Result then
      begin
        Result := writeChar(port, COMMAND_SYNC, nibble_high);
        byte_low := (nibble_high shl 4) or nibble_low;
        timeSinceInput := MAKEWORD(byte_low, byte_high);
      end;
    end;
  end;
end;

function getMuxValue(port: Word; var muxValue: Byte): Boolean;
var
  nibble_low, nibble_high: Byte;
begin
  Result := writeChar(port, COMMAND_GET_MUX_VALUE, nibble_low);
  if Result then
  begin
    Result := writeChar(port, COMMAND_SYNC, nibble_high);
    muxValue := (nibble_high shl 4) or nibble_low;
  end;
end;

function input(port: Word; channel: Byte; var value: Byte): Boolean;
var
  nibble_low, nibble_high: Byte;
begin
  Result := writeChar(port, COMMAND_INPUT, nibble_low);
  if Result then
  begin
    Result := writeChar(port, Chr(channel), nibble_low);
    if Result then
    begin
      Result := writeChar(port, COMMAND_SYNC, nibble_high);
      value := (nibble_high shl 4) or nibble_low;
    end;
  end;
end;

function bulkInput(port: Word; channels: array of Byte; var values: array of Byte): Boolean;
var
  i, nibble_low, nibble_high: Byte;
begin
  Result := writeChar(port, COMMAND_BULK_INPUT, nibble_low);
  if Result then
  begin
    for i := 0 to Length(values) - 1 do
    begin
      Result := writeChar(port, Chr(channels[i]), nibble_low);
      if Result then
      begin
        Result := writeChar(port, COMMAND_SYNC, nibble_high);
        values[i] := (nibble_high shl 4) or nibble_low;
      end
      else
        Break;
    end;
    if Result then
      Result := writeChar(port, COMMAND_SYNC, nibble_low);
  end;
end;

function setOutputStartcode(port: Word; value: Byte): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_SET_OUTPUT_STARTCODE, dummy);
  if Result then
    Result := writeChar(port, Chr(value), dummy);
end;

function output(port: Word; channel: Byte; value: Byte): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_OUTPUT, dummy);
  if Result then
  begin
    Result := writeChar(port, Chr(channel), dummy);
    if Result then
      Result := writeChar(port, Chr(value), dummy);
  end;
end;

function bulkOutput(port: Word; channels: array of Byte; values: array of Byte): Boolean;
var
  i, dummy: Byte;
begin
  Result := writeChar(port, COMMAND_BULK_OUTPUT, dummy);
  if Result then
  begin
    for i := 0 to Length(values) - 1 do
    begin
      Result := writeChar(port, Chr(channels[i]), dummy);
      if Result then
        Result := writeChar(port, Chr(values[i]), dummy)
      else
        Break;
    end;
    if Result then
      Result := writeChar(port, COMMAND_SYNC, dummy);
  end;
end;

function setMassOutputLimits(port: Word; minChannel, maxChannel: Byte): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_SET_MASS_OUTPUT_LIMITS, dummy);
  if Result then
  begin
    Result := writeChar(port, Chr(minChannel), dummy);
    if Result then
      Result := writeChar(port, Chr(maxChannel), dummy);
  end;
end;

function massOutput(port: Word; values: array of Byte): Boolean;
var
  i, dummy: Byte;
begin
  Result := writeChar(port, COMMAND_MASS_OUTPUT, dummy);
  if Result then
  begin
    for i := 0 to Length(values) - 1 do
    begin
      Result := writeChar(port, Chr(values[i]), dummy);
      if not Result then
        Break;
    end;
  end;
end;

function enableMux(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_ENABLE_MUX, dummy);
end;

function disableMux(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_DISABLE_MUX, dummy);
end;

function forceMuxOff(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_FORCE_MUX_OFF, dummy);
end;

function noForceMuxOff(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_NO_FORCE_MUX_OFF, dummy);
end;

function invertedMux(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_INVERTED_MUX, dummy);
end;

function nonInvertedMux(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_NON_INVERTED_MUX, dummy);
end;

function enableMerge(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_ENABLE_MERGE, dummy);
end;

function disableMerge(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_DISABLE_MERGE, dummy);
end;

function finish(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_FINISH, dummy);
end;

function readEEPROM(port: Word): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_READ_EEPROM, dummy);
end;

function writeEEPROM(port: Word; inputEnabled, outputEnabled, muxEnabled, mergeEnabled: Boolean): Boolean;
var
  dummy: Byte;
begin
  Result := writeChar(port, COMMAND_WRITE_EEPROM, dummy);
  if Result then
  begin
    if inputEnabled then
      Result := writeChar(port, COMMAND_ENABLE_INPUT, dummy)
    else
      Result := writeChar(port, COMMAND_DISABLE_INPUT, dummy);
    if Result then
    begin
      if outputEnabled then
        Result := writeChar(port, COMMAND_ENABLE_OUTPUT, dummy)
      else
        Result := writeChar(port, COMMAND_DISABLE_OUTPUT, dummy);
      if Result then
      begin
        if muxEnabled then
          Result := writeChar(port, COMMAND_ENABLE_MUX, dummy)
        else
          Result := writeChar(port, COMMAND_DISABLE_MUX, dummy);
        if Result then
        begin
          if mergeEnabled then
            Result := writeChar(port, COMMAND_ENABLE_MERGE, dummy)
          else
            Result := writeChar(port, COMMAND_DISABLE_MERGE, dummy);
        end;
      end;
    end;
  end;
end;

function echo(port: Word; echoRequest: Byte; var echoResult: Byte): Boolean;
var
  nibble_low, nibble_high: Byte;
begin
  Result := writeChar(port, COMMAND_ECHO, nibble_low);
  if Result then
  begin
    Result := writeChar(port, Chr(echoRequest), nibble_low);
    if Result then
    begin
      Result := writeChar(port, COMMAND_SYNC, nibble_high);
      echoResult := (nibble_high shl 4) or nibble_low;
    end;
  end;
end;

end.
