unit ueepromhelper;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, Buttons;

type
  TfEEPROMHelper = class(TForm)
    gbSurvey: TGroupBox;
    lMultiplexer: TLabel;
    lMerge: TLabel;
    lOutput: TLabel;
    lInput: TLabel;
    pOutput: TPanel;
    rbOutputYes: TRadioButton;
    rbOutputNo: TRadioButton;
    pInput: TPanel;
    rbInputYes: TRadioButton;
    rbInputNo: TRadioButton;
    pMux: TPanel;
    rbMultiplexerYes: TRadioButton;
    rbMultiplexerNo: TRadioButton;
    pMerge: TPanel;
    rbMergeYes: TRadioButton;
    rbMergeNo: TRadioButton;
    lOutputChannels: TLabel;
    eOutputChannels: TEdit;
    udOutputChannels: TUpDown;
    lMultiplexerAddress: TLabel;
    eMultiplexerAddress: TEdit;
    udMultiplexerAddress: TUpDown;
    lEEPROMHelper: TLabel;
    gbAddressing: TGroupBox;
    lAddressingOutput: TLabel;
    lAddressesOutput: TLabel;
    lAddressingMultiplexer: TLabel;
    lAddressesMultiplexer: TLabel;
    lAddressingMerge: TLabel;
    lAddressesMerge: TLabel;
    bbStore: TBitBtn;
    bbCancel: TBitBtn;
    procedure Change(Sender: TObject);
    procedure bbStoreClick(Sender: TObject);
  private
    { Private declarations }
    procedure recalculate;
  public
    { Public declarations }
  end;

var
  fEEPROMHelper: TfEEPROMHelper;

implementation

uses upartest;

{$R *.dfm}

procedure TfEEPROMHelper.recalculate;
var
  outputChannels: Byte;
begin
  lMultiplexerAddress.Enabled  := rbMultiplexerYes.Checked;
  eMultiplexerAddress.Enabled  := lMultiplexerAddress.Enabled;
  udMultiplexerAddress.Enabled := lMultiplexerAddress.Enabled;
  if not(lMultiplexerAddress.Enabled) then
    udMultiplexerAddress.Position := 0;

  // only show output choice if both multiplexer and merge are disabled,
  // otherwise output would be forced to be enabled
  lOutput.Enabled     := rbMultiplexerNo.Checked and rbMergeNo.Checked;
  rbOutputYes.Enabled := lOutput.Enabled;
  rbOutputNo.Enabled  := lOutput.Enabled;
  if not(lOutput.Enabled) then
    rbOutputYes.Checked := True;

  lOutputChannels.Enabled  := rbMultiplexerYes.Checked or rbMergeYes.Checked or rbOutputYes.Enabled;
  eOutputChannels.Enabled  := lOutputChannels.Enabled;
  udOutputChannels.Enabled := lOutputChannels.Enabled;
  if not(lOutputChannels.Enabled) then
    udOutputChannels.Position := 255;
  if lMultiplexerAddress.Enabled then
  begin
    outputChannels := udMultiplexerAddress.Position + 8;
    if udOutputChannels.Position < outputChannels then
      udOutputChannels.Position := outputChannels;
    udOutputChannels.Min := outputChannels;
  end
  else
    udOutputChannels.Min := 0;

  // only show input choice if merge mode isn't enabled,
  // otherwise input would be forced to be enabled
  lInput.Enabled     := rbMergeNo.Checked;
  rbInputYes.Enabled := lInput.Enabled;
  rbInputNo.Enabled  := lInput.Enabled;
  if not(lInput.Enabled) then
    rbInputYes.Checked := True;

  bbStore.Enabled :=
    (rbMultiplexerYes.Checked or rbMultiplexerNo.Checked)
    and
    (rbMergeYes.Checked or rbMergeNo.Checked)
    and
    (rbOutputYes.Checked or rbOutputNo.Checked)
    and
    (rbInputYes.Checked or rbInputNo.Checked);
  gbAddressing.Visible :=
    bbStore.Enabled
    and
    rbOutputYes.Checked;
  if gbAddressing.Visible then
  begin
    lAddressingOutput.Visible      := udOutputChannels.Position > 0;
    lAddressesOutput.Visible       := lAddressingOutput.Visible;

    lAddressingMultiplexer.Visible := rbMultiplexerYes.Checked;
    lAddressesMultiplexer.Visible  := lAddressingMultiplexer.Visible;

    lAddressingMerge.Visible       := rbMergeYes.Checked;
    lAddressesMerge.Visible        := lAddressingMerge.Visible;

    if udOutputChannels.Position > 1 then
      lAddressesOutput.Caption := Format('000 up to %.3d', [udOutputChannels.Position - 1])
    else
      lAddressesOutput.Caption := '000';
    lAddressesMultiplexer.Caption := Format('%.3d up to %.3d', [udMultiplexerAddress.Position, udMultiplexerAddress.Position + 7]);
    lAddressesMerge.Caption := Format('%.3d up to %.3d', [udOutputChannels.Position, udOutputChannels.Position + 245]);
  end;
end;

procedure TfEEPROMHelper.Change(Sender: TObject);
begin
  recalculate;
end;

procedure TfEEPROMHelper.bbStoreClick(Sender: TObject);
begin
  fParTest.eMuxAddress.Text       := IntToStr(udMultiplexerAddress.Position);
  fParTest.bMuxAddressClick(Sender);
  fParTest.eOutputCount.Text      := IntToStr(udOutputChannels.Position);
  fParTest.bOutputCountClick(Sender);
  fParTest.chEnableInput.Checked  := rbInputYes.Checked;
  fParTest.chEnableOutput.Checked := rbOutputYes.Checked;
  fParTest.chEnableMux.Checked    := rbMultiplexerYes.Checked;
  fParTest.chEnableMerge.Checked  := rbMergeYes.Checked;
  fParTest.bWriteEEPROMClick(Sender);
end;

end.
